#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <asm/types.h>
#include "smu.h"

int fd;
struct smu_user_cmd_hdr *chdr;
struct smu_user_reply_hdr *rhdr;
unsigned char *cdata, *rdata;
unsigned char buffer[1024];

static int getservermode(void)
{
       int rc;

        chdr->cmdtype = SMU_CMDTYPE_SMU;
        chdr->cmd = SMU_CMD_POWER_EVENTS_COMMAND;
        chdr->data_len = 1;
        cdata[0] = SMU_PWR_GET_POWERUP_EVENTS;
        rc = write(fd, chdr, sizeof(*chdr) + 1);
        if (rc < 0) {
                perror("error writing command");
                return rc;
        }
        rc = read(fd, rhdr, 1024);
        if (rc < 0) {
                perror("error reading reply");
                return rc;
        }
        //printf("getservermode status: %d, reply_len: %d\n",
        //       rhdr->status, rhdr->reply_len);
	if (rhdr->status == 0 && rhdr->reply_len >= 2)
		printf("SMU server mode is %s [events: 0x%02x 0x%02x]\n",
			(rdata[1] & SMU_PWR_WAKEUP_AC_INSERT) ? "ON" : "OFF",
			rdata[0], rdata[1]);
        return 0;
}

static int setservermode(int state)
{
	int rc;

	printf("Setting SMU server mode to %s...\n",
		state ? "ON" : "OFF");

	chdr->cmdtype = SMU_CMDTYPE_SMU;
	chdr->cmd = SMU_CMD_POWER_EVENTS_COMMAND;
	chdr->data_len = 3;
	cdata[0] = state ? SMU_PWR_SET_POWERUP_EVENTS :
			   SMU_PWR_CLR_POWERUP_EVENTS;
	cdata[1] = 0x0;
	cdata[2] = SMU_PWR_WAKEUP_AC_INSERT;
	rc = write(fd, chdr, sizeof(*chdr) + 3);
	if (rc < 0) {
		perror("error writing command");
		return rc;
	}
	rc = read(fd, rhdr, 1024);
	if (rc < 0) {
		perror("error reading reply");
		return rc;
	}
	//printf("servermode status: %d, reply_len: %d\n",
	//       rhdr->status, rhdr->reply_len);
	return 0;
}

int main(int argc, char **argv)
{
	chdr = (struct smu_user_cmd_hdr *)buffer;
	rhdr = (struct smu_user_reply_hdr *)buffer;
	cdata = &buffer[sizeof(*chdr)];
	rdata = &buffer[sizeof(*rhdr)];

	fd = open("/dev/smu", O_RDWR);
	if (fd == -1) {
		perror("can't open /dev/smu");
		return -1;
	}
	if (argc < 2) {
		getservermode();
		return 0;
	}
	setservermode(atoi(argv[1]));
	getservermode();
	return 0;
}

